'Gaak, das Entenprogramm - So fing es jedenfalls an, denn zu Beginn war nur
'ein Schaltmodul zum Ein/Ausschalten eines Soundmoduls einer schwimmenden
'RC-gesteuerten Ente gefragt. Inzwischen ist es zu einem 4-fach-Schaltmodul
'mutiert, das auf wiederholtes schnelles Umschalten des Steuerknppels eines
'anderweitig genutzten Kanals (Antrieb, Ruder) reagiert.

'Die Funktion ist Folgendermaen:

'Durch schnelles Schalten des Steuerknppels von einer Endlage (kurzer
'Kanalimpuls) zur anderen Endlage (langer Kanalimpuls) wird ein Zhler erhht.
'Bleiben weitere Zhlvorgnge aus, so wird die dem Zhlerstand zugehrige
'Schalthandlung durchgefhrt.

'Anschlussbelegung des Tiny13:

'Pin 1: Reset, Widerstand 4k7...10k nach Pin 8 (VCC)
'Pin 2: PB3, Ausgang 3
'Pin 3: PB4, Ausgang 4
'Pin 4: GND, Masse
'Pin 5: PB0, Ausgang 1
'Pin 6: PB1, Impulseingang vom RC-Empfnger (ber Schutzwiderstand 1k)
'Pin 7: PB2, Ausgang 2
'Pin 8: VCC, Versorgungsspannung, Kerko 100n nach GND, Reset-Widerstand

'Das Programm wurde bewusst ohne Interrupts geschrieben, da in Bascom die
'Interrupts aufgrund der kompletten Registersicherung unntig lange dauern.

'Die Fusebits des Tiny13 sind folgendermaen einzustellen:
'High-Fuse: 0xFB, Low-Fuse: 0x6A
'Dabei ist eigentlich nur Brown-Out auf 2,7 Volt eingestellt. Alle anderen
'Fuses sind im Auslieferungszustand des Tiny13 belassen worden. Die
'Takteinstellung erfolgt nicht per Fusebits, sondern per Software.
'Dazu muss der interne RC-Oszillator mit 9,6 MHz aktiviert sein. Der Zustand
'der CKDIV8-Fuse ist egal, der Clock-Prescaler wird per Software berschrieben.


'einige vordefinierte Parameter:
Const Zeitlimit = 30               'Zeitlimit zum Umschalten (in 20 ms)
Const Zaehlzeit = 50               'Zeitlimit fr Zhlimpulse (in 20 ms)
Const Maxposi = 130                'Schwellwert langer Impuls (75 bis 150)
Const Minposi = 95                 'Schwellwert kurzer Impuls (75 bis 150)
Const Maxlimit = 180               'maximal erlaubte Impulsbreite
Const Minlimit = 50                'minimal erlaubte Impulsbreite
Const Auszeit = 65535              'Zeit bis zum Ausschalten bei Senderausfall

'Bascom-spezifische Deklarationen:
$regfile = "attiny13.dat"          'Portnamen vereinbaren
$crystal = 4800000                 'Takt vereinbaren
$hwstack = 32                      'mach'mer immer so
$swstack = 8                       'mach'mer immer so
$framesize = 16                    'mach'mer immer so und keiner wei warum

'Den Portpins erstmal sinnvolle/sinnlose Namen geben:
Ausg1 Alias Portb.0                'Steuerausgang 1 fr Soundmodul
Impuls Alias Pinb.1                'Impulseingang
Ausg2 Alias Portb.2                'Steuerausgang 2 fr Soundmodul
Ausg3 Alias Portb.3                'Testausgang Max-Wert
Ausg4 Alias Portb.4                'Testausgang Min-Wert

Dim Zsalt As Byte                  'Zeitszempel Impulsbeginn
Dim Posi As Byte                   'Positionswert Kanalimpuls
Dim Zeit1 As Byte                  'CountDown 1 (Zeitfenster Knppelbewegung)
Dim Zeit2 As Byte                  'CountDown 2 (Zeitfenster Zhlvorgang)
Dim Nummer As Byte                 'Zhler fr Anzahl der Knppelbewegungen
Dim Wachhund As Word               'Zeitzhler fr Senderausfall
Dim Merker As Byte                 'Pool fr Bitvariablen
  Ida Alias Merker.0               'Merker, ein neuer gltiger Impuls ist da

'Programmstart, Reset-Routine:
'Tiny13 auf 4,8 MHz umschalten:
Clkpr = 128                        'Taktvorteiler Schreibschutz aufheben
Clkpr = 1                          'Taktvorteiler auf 1:2
Clkpr = 1                          'einstellen

'PB1 (Pin 6) wird der Impulseingang, alles andere werden Ausgnge
Portb = &B00010                    'Ports erstmal auf L-Pegel, PullUp 1 ein
Ddrb = &B11101                     'PB2 als Eingang, Rest als Ausgeng

'Timer einschalten, es werden keine Pin-Sonderfunktionen und auch kein
'Interrupt verwendet. Der Timer wird von der Hauptschleife ausgelesen und
'gelscht.
Tccr0b = 3                         'Timer mit Vorteiler 1:64 starten

'Hauptschleife des Programms
Do                                 'Beginn der Endlos-Schleife
  If Impuls = 0 Then               'nur bei L-Pegel am Impuls-Eingang
    Posi = Timer0                  'Zeitstempel holen,
    Posi = Posi - Zsalt            'Differenz zur vorherigen Runde ermitteln,
    Zsalt = Timer0                 'und alten Zeitstempel nachziehen
    'Die Abfrage erfolgt ja nur whrend des L-Pegels. Die Differenz zwischen
    'neuem und altem Zeitstempel kann nur dann einen plausiblen Wert (50...180)
    'erreichen, wenn zwischen beiden Scanns ein H-Pegel (0,67...2,4ms) auftrat.
    'Dies wird jetzt berprft:
    Set Ida                        'erstmal einen gltigen Impuls vermuten,
    'also von einer korrekten Impulsbreite ausgehen, dann die Grenzwerte der
    'Impulsbreite berprfen und bei berschreitung fr ungltig erklren:
    If Posi > Maxlimit Then Reset Ida       'Impuls zu lang, ungltig
    If Posi < Minlimit Then Reset Ida       'Impuls zu kurz, ungltig
    'Der Merker "Ida" entscheidet nun darber, ob ein gltiger Kanalimpuls
    'empfangen wurde oder nicht.

    If Ida = 1 Then                'falls ein Impuls im erlaubten Bereich kam:
      'Es wurde ein gltiger Impuls empfangen. Da dies im regelmigen Abstand
      'von etwa 20ms (Telegrammabstand des Senders) der Fall ist, kann dieser
      '"Takt" gleich als Zeittakt zum Herunterzhlen der Timeouts missbraucht
      'werden. Nur aktive Timeouts (ungleich 0) werden heruntergezhlt.
      If Zeit1 > 0 Then Decr Zeit1 'Countdown 1 herunterzhlen, falls aktiv
      If Zeit2 > 0 Then Decr Zeit2 'Countdown 2 herunterzhlen, falls aktiv
      'Die Count-Down-Zhler sind nun fertig. Jetzt wird der Wachhund gefttert
      'und gestreichelt, damit er die Ausgnge nicht abschaltet, denn es wurde
      'ja ein gltiger Impuls empfangen, der Sender lebt also noch:
      Wachhund = Auszeit           'Senderausfall-TimeOut auf Startwert setzen
      'Jetzt wird geprft, ob die Impulsbreite den Schwellwert einer
      'Schaltposition berschreitet:

      If Posi < Minposi Then       'wenn Impuls krzer als Schwellwert ist:
        'Hier wird nur das Zeitfenster zum Schalten der Gegenrichtung geffnet.
        Zeit1 = Zeitlimit          'Knppel-CountDown auf Startwert
      End If                       'Ende (Impuls ist kleiner als Schwellwert)

      If Posi > Maxposi Then       'wenn Impuls lnger als Schwellwert ist:
        'Hier wird nur innerhalb des noch nicht abgelaufenen Knppel-
        'Zeitfensters auf langen Kanalimpuls reagiert:

        If Zeit1 > 0 Then          'ist Knppel-CountDown noch aktiv, dann
          Zeit1 = 0                'und CountDown entwerten
          Zeit2 = Zaehlzeit        'Zhl-CountDown auf Startwert
          Incr Nummer              'Job-Nummer hochzhlen
        End If                     'Ende (Knppel-CountDown noch aktiv)

      End If                       'Ende (Impuls ist grer als Schwellwert)

      If Zeit2 = 1 Then            'wenn das Zhl-Zeitfenster abluft, dann
        'Es wurde verpasst, rechtzeitig weiter zu zhlen, somit ist der
        'aktuelle Zhlerstand gltig und kann ausgewertet werden:
        If Nummer = 3 Then Toggle Ausg1       'Ausgang 1 umschalten
        If Nummer = 4 Then Toggle Ausg2       'Ausgang 2 umschalten
        If Nummer = 5 Then Toggle Ausg3       'Ausgang 3 umschalten
        If Nummer = 6 Then Toggle Ausg4       'Ausgang 4 umschalten
        If Nummer = 7 Then Portb = &B00010       'alle Ausgnge ausschalten
        If Nummer = 8 Then Portb = &B11111       'alle Ausgnge einschalten
        Nummer = 0                 'Job-Nummer entwerten
        'Zeit2 wird vor der nchsten Abfrage bereits vermindert und erreicht
        'somit auch ohne weiteres Zutun 0, was weitere Zhlvorgnge blockiert.
      End If                       'Ende (wenn das Zhl-Zeitfenster abluft)

    End If                         'Ende (gltiger Impuls)
  End If                           'Ende (L-Pegel liegt an)
  Decr Wachhund                    'Timeout Impulsausfall herunterzhlen
  If Wachhund = 0 Then Portb = &B00010       'angelaufen? - Ports ausschalten
Loop                               'zurck zum Beginn der Endlos-Schleife