'Gaak, das Entenprogramm...

'Ein Schaltmodul zum Einschalten eines Soundmoduls und Ein/Ausschalten eines
'Wechselblinkers einer schwimmenden RC-gesteuerten Ente.

'Die Funktion ist Folgendermaen:

'Durch schnelles Schalten des Steuerknppels von der Neutralstellung zu einer
'Endlage (kurzer oder langer Kanalimpuls) wird eine Aktion ausgelst.
'Mit "kurz" wird ein Wechselblinker ein- oder ausgeschaltet, der auf die
'Ausgnge 1 und 2 (Pin 5 und Pin 7) wirkt.
'Mit "lang" wird ein Impuls auf Ausgang 4 (Pin 3) gelegt, mit dem ein Relais
'oder Optokoppler zum Starten des Soundmoduls geschaltet werden kann.

'Anschlussbelegung des Tiny13:

'Pin 1: Reset, Widerstand 4k7...10k nach Pin 8 (VCC)
'Pin 2: PB3, Ausgang 3
'Pin 3: PB4, Ausgang 4
'Pin 4: GND, Masse
'Pin 5: PB0, Ausgang 1
'Pin 6: PB1, Impulseingang vom RC-Empfnger (ber Schutzwiderstand 1k)
'Pin 7: PB2, Ausgang 2
'Pin 8: VCC, Versorgungsspannung, Kerko 100n nach GND, Reset-Widerstand

'Das Programm wurde bewusst ohne Interrupts geschrieben, da in Bascom die
'Interrupts aufgrund der kompletten Registersicherung unntig lange dauern.
'Der Senderausfall-TimeOut wurde durch Zhlen der Anzahl der Hauptschleifen-
'durchlufe ohne gltigen Kanalimpuls realisiert, die Kanalimpulsdauer durch
'Zeitstempel des freilaufenden Timers (Software ICP), alle anderen Zeiten
'durch Zhlen der vom Sender und Empfnger erzeugten Kanalimpulse.

'Das Programm wurde bewusst in Bascom geschrieben und ppig kommentiert, da
'es als Hilfe beim Einstieg in die AVR-Programmierung mit Bascom dienen soll.

'Die Fusebits des Tiny13 sind folgendermaen einzustellen:
'High-Fuse: 0xFB, Low-Fuse: 0x6A
'Dabei ist eigentlich nur Brown-Out auf 2,7 Volt eingestellt. Alle anderen
'Fuses sind im Auslieferungszustand des Tiny13 belassen worden. Die
'Takteinstellung erfolgt nicht per Fusebits, sondern per Software.
'Dazu muss der interne RC-Oszillator mit 9,6 MHz aktiviert sein. Der Zustand
'der CKDIV8-Fuse ist egal, der Clock-Prescaler wird per Software berschrieben.


'einige vordefinierte Parameter:
Const Sperrzeit = 50               'Anzahl der ignorierten Impulse bei Start
Const Zeitlimit = 30               'Zeitlimit zum Umschalten (in 20 ms)
Const Zaehlzeit = 50               'Zeitlimit fr Zhlimpulse (in 20 ms)
Const Maxposi = 130                'Schwellwert langer Impuls (75 bis 150)
Const Mitposu = 105                'untere Grenze Mittenposition
Const Mitposo = 120                'obere Grenze Mittenposition
Const Minposi = 95                 'Schwellwert kurzer Impuls (75 bis 150)
Const Maxlimit = 180               'maximal erlaubte Impulsbreite
Const Minlimit = 50                'minimal erlaubte Impulsbreite
Const Relzeit = 25                 'Haltezeit fr Sound-Relais (in 20 ms)
Const Blinkzeit = 25               'Toggelzeit fr Wechselblinker (in 20 ms)
'Per Simulator wird ermittelt, wieviele Rechentakte die Hauptschleife
'mindestens braucht, wenn kein Impulsende detektiert wird. Das Ergebnis
'wird hier eingetragen und ist fr den Senderausfall-Timeout zustndig:
Const Leerlauf = 92                '92 CPU-Takte je leeren Schleifendurchlauf
Const Txaus = 25                   'Senderausfall-Reaktionszeit in ms

Const Auszeit = 4800 * Txaus / Leerlauf       'Startwert Senderausfallzhler

'Bascom-spezifische Deklarationen:
$regfile = "attiny13.dat"          'Portnamen vereinbaren
$crystal = 4800000                 'Takt vereinbaren
$hwstack = 4                       'keine Interrupts, keine Unterprogramme
$swstack = 4                       'keine ByVal-bergabewerte ntig
$framesize = 4                     'kein Rechenbereich fr Funktionen ntig

'Den Portpins erstmal sinnvolle/sinnlose Namen geben:
Ausg1 Alias Portb.0                'Steuerausgang 1 fr Soundmodul
Impuls Alias Pinb.1                'Impulseingang
Ausg2 Alias Portb.2                'Steuerausgang 2 fr Soundmodul
Ausg3 Alias Portb.3                'Testausgang Max-Wert
Ausg4 Alias Portb.4                'Testausgang Min-Wert

'Variablen einrichten:
Dim Zsalt As Byte                  'Zeitszempel Impulsbeginn
Dim Posi As Byte                   'Positionswert Kanalimpuls
Dim Zeit1 As Byte                  'CountDown 1 (Zeitfenster Knppelbewegung)
Dim Zeit2 As Byte                  'CountDown 2 (Zeitfenster Zhlvorgang)
Dim Kunum As Byte                  'Zhler fr Anzahl der Knppelbewegungen k
Dim Lanum As Byte                  'Zhler fr Anzahl der Knppelbewegungen l
Dim Blinker As Byte                'Zhler fr Blinkgeber
Dim Relaus As Byte                 'Zhler fr Soundimpuls
Dim Wachhund As Word               'Zeitzhler fr Senderausfall
Dim Softstart As Byte              'Unterdrcken der ersten Kanalimpulse
Dim Merker As Byte                 'Pool fr Bitvariablen
  Ida Alias Merker.0               'Merker, ein neuer gltiger Impuls ist da
  Blan Alias Merker.1              'Merker, Blinker ist an

'Programmstart, Reset-Routine:
'Tiny13 auf 4,8 MHz umschalten:
Clkpr = 128                        'Taktvorteiler Schreibschutz aufheben
Clkpr = 1                          'Taktvorteiler auf 1:2
Clkpr = 1                          'einstellen

'PB1 (Pin 6) wird der Impulseingang, alles andere werden Ausgnge
Portb = &B00010                    'Ports erstmal auf L-Pegel, PullUp 1 ein
Ddrb = &B11101                     'PB2 als Eingang, Rest als Ausgeng

'Timer einschalten, es werden keine Pin-Sonderfunktionen und auch kein
'Interrupt verwendet. Der Timer wird von der Hauptschleife ausgelesen und
'gelscht.
Tccr0b = 3                         'Timer mit Vorteiler 1:64 starten
Softstart = Sperrzeit              'erste 50 Kanalimpulse ignorieren

'Hauptschleife des Programms
Do                                 'Beginn der Endlos-Schleife
  If Impuls = 0 Then               'nur bei L-Pegel am Impuls-Eingang
    Posi = Timer0                  'Zeitstempel holen,
    Posi = Posi - Zsalt            'Differenz zur vorherigen Runde ermitteln,
    Zsalt = Timer0                 'und alten Zeitstempel nachziehen
    'Die Abfrage erfolgt ja nur whrend des L-Pegels. Die Differenz zwischen
    'neuem und altem Zeitstempel kann nur dann einen plausiblen Wert (50...180)
    'erreichen, wenn zwischen beiden Scanns ein H-Pegel (0,67...2,4ms) auftrat.
    'Dies wird jetzt berprft:
    Set Ida                        'erstmal einen gltigen Impuls vermuten,
    'also von einer korrekten Impulsbreite ausgehen, dann die Grenzwerte der
    'Impulsbreite berprfen und bei berschreitung fr ungltig erklren:
    If Posi > Maxlimit Then Reset Ida       'Impuls zu lang, ungltig
    If Posi < Minlimit Then Reset Ida       'Impuls zu kurz, ungltig
    If Softstart > 0 Then          'Einschaltsperre nach Start und TX-Ausfall
      Decr Softstart               'Sperrzhler runterzhlen
      Reset Ida                    'Impuls entwerten (ignorieren)
    End If

    'Der Merker "Ida" entscheidet nun darber, ob ein gltiger Kanalimpuls
    'empfangen wurde oder nicht:

    If Ida = 1 Then                'falls ein Impuls im erlaubten Bereich kam:
      'Es wurde ein gltiger Impuls empfangen. Da dies im regelmigen Abstand
      'von etwa 20ms (Telegrammabstand des Senders) der Fall ist, kann dieser
      '"Takt" gleich als Zeittakt zum Herunterzhlen der Timeouts missbraucht
      'werden. Nur aktive Timeouts (ungleich 0) werden heruntergezhlt.
      If Zeit1 > 0 Then Decr Zeit1 'Countdown 1 herunterzhlen, falls aktiv
      If Zeit2 > 0 Then Decr Zeit2 'Countdown 2 herunterzhlen, falls aktiv

      'Die Count-Down-Zhler sind nun fertig. Jetzt wird der Wachhund gefttert
      'und gestreichelt, damit er die Ausgnge nicht abschaltet, denn es wurde
      'ja ein gltiger Impuls empfangen, der Sender lebt also noch:
      Wachhund = Auszeit           'Senderausfall-TimeOut auf Startwert setzen

      'Jetzt wird geprft, ob die Impulsbreite innerhalb der Neutralposition
      'liegt:

      If Posi < Mitposo Then       'wenn Impuls krzer als obere Mitte ist:
        If Posi > Mitposu Then     'wenn Impuls lnger als untere Mitte ist:
'Set Ausg2
          'Hier wird nur das Zeitfenster zum Schalten der Gegenrichtung geffnet.
          Zeit1 = Zeitlimit        'Knppel-CountDown auf Startwert
'Else
'Reset Ausg2
        End If                     'Ende (Impuls ist lnger als obere Mitte)
'Else
'Reset Ausg2
      End If                       'Ende (Impuls ist krzer als untere Mitte)

      'Jetzt wird geprft, ob die Impulsbreite den Schwellwert einer
      'Schaltposition berschreitet:

      If Posi < Minposi Then       'wenn Impuls krzer als Schwellwert ist:
        'Hier wird nur innerhalb des noch nicht abgelaufenen Knppel-
        'Zeitfensters auf kurzen Kanalimpuls reagiert:

        If Zeit1 > 0 Then          'ist Knppel-CountDown noch aktiv, dann
          Incr Kunum               'Job-Nummer (kurze Richtung) hochzhlen,
          Zeit1 = 0                'CountDown entwerten und
          Zeit2 = Zaehlzeit        'Zhl-CountDown auf Startwert
        End If                     'Ende (Knppel-CountDown noch aktiv)

      End If                       'Ende (Impuls ist kleiner als Schwellwert)

      If Posi > Maxposi Then       'wenn Impuls lnger als Schwellwert ist:
        'Hier wird nur innerhalb des noch nicht abgelaufenen Knppel-
        'Zeitfensters auf langen Kanalimpuls reagiert:

        If Zeit1 > 0 Then          'ist Knppel-CountDown noch aktiv, dann
          Incr Lanum               'Job-Nummer (lange Richtung) hochzhlen,
          Zeit1 = 0                'CountDown entwerten und
          Zeit2 = Zaehlzeit        'Zhl-CountDown auf Startwert
        End If                     'Ende (Knppel-CountDown noch aktiv)

      End If                       'Ende (Impuls ist grer als Schwellwert)

'Hier werden die Schalthandlungen ausgefhrt, dabei sind, Set, Reset, Toggle
'und direkte Portzuweisungen zulssig:

      If Zeit2 = 1 Then            'wenn das Zhl-Zeitfenster abluft, dann
        'Es wurde verpasst, rechtzeitig weiter zu zhlen, somit ist der
        'aktuelle Zhlerstand gltig und kann ausgewertet werden:
        If Kunum = 1 Then
          Toggle Blan              'Blinker umschalten (ein/aus)
          Blinker = Blinkzeit      'Blinktimer auf Startwert
          Ausg1 = Blan             'Startzustand fr
          Reset Ausg2              'Blinker
        End If

        If Lanum = 1 Then
          Set Ausg4                'Relais ein
          Relaus = Relzeit         'Ausschalt-Termin setzen
        End If

        Kunum = 0                   'Job-Nummer entwerten
        Lanum = 0                  'Job-Nummer entwerten
        'Zeit2 wird vor der nchsten Abfrage bereits vermindert und erreicht
        'somit auch ohne weiteres Zutun 0, was weitere Zhlvorgnge blockiert.
      End If                       'Ende (wenn das Zhl-Zeitfenster abluft)

'Hier werden die Zeiten fr Relais und Blinker abgearbeitet:
      If Relaus > 0 Then           'wenn Relais-Timeout luft, dann
        Decr Relaus                'Relais-Timeout herunterzhlen
        If Relaus = 0 Then Reset Ausg4       'wenn unten, dann aus
      End If                       'Ende (Relais-Timeout)

      If Blan = 1 Then             'wenn der Blinker aktiv ist, dann
        Decr Blinker               'Blinkzeit herunterzhlen
        If Blinker = 0 Then        'wenn Blinkzeit um, dann
          Blinker = Blinkzeit      'Blinkzeit auf Startwert
          Toggle Ausg1             'beide Blinkausgnge
          Toggle Ausg2             'umschalten
        End If                     'Ende (Blinkzeit um)
      End If                       'Ende (Blinker aktiv)

    End If                         'Ende (gltiger Impuls)
  End If                           'Ende (L-Pegel liegt an)
  Decr Wachhund                    'Timeout Impulsausfall herunterzhlen
  If Wachhund = 0 Then
    Portb = &B00010                'angelaufen? - alle Ports ausschalten
    Merker = 0                     'alle Merker ausschalten
    Softstart = Sperrzeit          'die nchsten Kanalimpulse ignorieren
  End If
Loop                               'zurck zum Beginn der Endlos-Schleife